# echo-artist.rb: Supporting library for echo-artist scripts 
#
# Copyright (C) 2008, Martin Sourada
# 
# This script is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this script; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

require "rexml/document"
include REXML

module EchoArtist
  ECHO_SIZES = ['16x16', '22x22', '24x24', '32x32', '48x48', '256x256', 'scalable']
  ECHO_BRANCHES_LEGACY = ['0.3.x', '0.3.2.x']
  ECHO_BRANCHES_AUTOTOOLS = ['master']
  ECHO_CONTEXTS = ['actions', 'animations', 'apps', 'categories', 'devices', 'emblems', 'emotes', 'intl', 'mimetypes', 'places', 'status']

  def check_context(context)
    # Check for correct context
    if !ECHO_CONTEXTS.include? context
      puts "Error: Unknown icon context '#{context}'. \nContext must be one of: "
      p ECHO_CONTEXTS
      return FALSE
    end

    return TRUE
  end

  def check_document_metadata(filename)
    svg = Document.new(File.new("#{filename}", 'r'))

    # Check for icon name
    element = svg.root.elements['//dc:title']
    if element
      title = element.text
      if !title
        puts "Error: Missing document metadata title (icon name)."
        return FALSE
      else
        iconname = File.basename(filename, ".svg")
        if title != iconname
          puts "Warning: Icon name in document metadata differs from file name."
          puts "Icon name in document metadata is: #{title}"
          puts "Icon file name (without extension) is: #{iconname}"
        end  
      end
    else
      puts "Error: Missing document metadata title (icon name)."
      return FALSE
    end

    # Check for icon context
    element = svg.root.elements['//dc:description']
    if element
      context = element.text
      if !context
        puts "Error: Missing document metadata description (icon context)."
        return FALSE
      else
        if !check_context(context)
          return FALSE
        end
      end
    else
      puts "Error: Missing document metadata description (icon context)."
      return FALSE
    end
    
    return TRUE
  end

  def check_layer_plate(filename)
    svg = Document.new(File.new("#{filename}", 'r'))

    # Check if plate layer is present and hidden
    plate_present = FALSE
    svg.root.each_element("//g[@inkscape:label='plate']") do |plate|
      plate_present = TRUE
      style = plate.attributes['style']
      if style != "display:none"
        puts "Error: Plate layer is not hidden."
        return FALSE
      end
    end

    if !plate_present
      puts "Error: Plate layer is missing."
      return FALSE
    end

    # Check for extra sizes/duplicates in plate layer
    plate_sizes = ['scalable']
    svg.root.each_element("//g[@inkscape:label='plate']/rect") do |icon|
      size = "#{icon.attributes['width']}x#{icon.attributes['height']}"
      if ECHO_SIZES.include? "#{size}"
        if plate_sizes.include? "#{size}"
          puts "Error: Size #{size} detected multiple times."
          return FALSE
	else
          plate_sizes += ["#{size}"]
        end
      else
        puts "Error: Unknown icon size present in plate layer: #{size}."
        return FALSE
      end
    end
    
    # Check for missing sizes in plate layer
    ECHO_SIZES.each do |icon|
      if !plate_sizes.include? icon
        puts "Warning: Size #{icon} missing from plate layer."
      end
    end

    # Check for scalable icon canvas
    scalable = svg.root.elements["//g[@inkscape:label='plate']/rect[@inkscape:label='scalable']"]
    if !scalable
      puts "Error: Missing element with inkscape:label='scalable' in plate layer."
      return FALSE
    end

    return TRUE
  end

  def check_layer_artwork(filename)
    svg = Document.new(File.new("#{filename}", 'r'))

    # Check if artwork layer is present
    artwork = svg.root.elements["//g[@inkscape:label='artwork']"]
    if !artwork
      puts "Error: Artwork layer is missing."
      return FALSE
    end

    # Check for scalable icon artwork
    scalable = svg.root.elements["//g[@inkscape:label='artwork']/g[@inkscape:label='scalable']"]
    if !scalable
      puts "Error: Missing object group with inkscape:label='scalable' in artwork layer."
      return FALSE
    end

    return TRUE
  end
end
